;; synergy/theme.jl
;;
;; Copyright (C) 2000 Nathan Froyd <froydnj@rose-hulman.edu>
;; 
;; Distributed under the terms of the GNU General Public License
;; version 2, which you should already have several copies of

(defgroup synergy "synergy theme"
  :group appearance)

(defvar synergy:title-font
  "-b&h-lucidatypewriter-medium-r-normal-*-*-100-*-*-m-*-iso8859-1")

; taken out until we figure how to draw titles with font-height
;(defcustom synergy:title-font
;  "-b&h-lucidatypewriter-medium-r-normal-*-*-100-*-*-m-*-iso8859-1"
;  "Title font for this theme."
;  :type font
;  :group (appearance synergy))

(defcustom synergy:squeeze-titlebar
  t
  "Whether or not to have a BeOS-type titlebar."
  :type boolean
  :group (appearance synergy))

(defcustom synergy:resizebar-height
  4
  "How big the resizebar is, including the bevels."
  :type number
  :range (4 . 10)
  :group (appearance synergy))

;;{{{ Colors

(defcustom synergy:edge-color
  (get-color "black")
  "Surrounding color for the whole frame."
  :type color
  :group (appearance synergy))

(defcustom synergy:background-color-nofocus
  (get-color "#3c1e00003850")
  "Background color for the unfocused titlebar."
  :type color
  :group (appearance synergy))

(defcustom synergy:foreground-color-nofocus
  (get-color "#e146df9b49a0")
  "Foreground color for the unfocused titlebar."
  :type color
  :group (appearance synergy))

(defcustom synergy:background-color-focus
  (get-color "#07530059707b")
  "Background color for the focused titlebar."
  :type color
  :group (appearance synergy))

(defcustom synergy:foreground-color-focus
  (get-color "#e146df9b49a0")
  "Foreground color for the focused titlebar."
  :type color
  :group (appearance synergy))

(defcustom synergy:resizebar-color-nofocus
  (get-color "gray40")
  "Color for the unfocused resizebar."
  :type color
  :group (appearance synergy))

(defcustom synergy:resizebar-color-focus
  (get-color "gray50")
  "Color for the focused resizebar."
  :type color
  :group (appearance synergy))

;;}}}
;;{{{ Bevels

(defgroup synergy-bevel "Bevel settings"
  :group (appearance synergy))

(defcustom synergy-bevel:red-highlight-percentage
  40
  "Percentage to add to obtain the highlight color on bevels."
  :type number
  :range (0 . 100)
  :group (appearance synergy synergy-bevel))

(defcustom synergy-bevel:green-highlight-percentage
  40
  "Percentage to add to obtain the highlight color on bevels."
  :type number
  :range (0 . 100)
  :group (appearance synergy synergy-bevel))

(defcustom synergy-bevel:blue-highlight-percentage
  40
  "Percentage to add to obtain the highlight color on bevels."
  :type number
  :range (0 . 100)
  :group (appearance synergy synergy-bevel))

(defcustom synergy-bevel:red-shadow-percentage
  30
  "Percentage to subtract to obtain the shadow color on bevels."
  :type number
  :range (0 . 100)
  :group (appearance synergy synergy-bevel))

(defcustom synergy-bevel:green-shadow-percentage
  30
  "Percentage to subtract to obtain the shadow color on bevels."
  :type number
  :range (0 . 100)
  :group (appearance synergy synergy-bevel))

(defcustom synergy-bevel:blue-shadow-percentage
  30
  "Percentage to subtract to obtain the shadow color on bevels."
  :type number
  :range (0 . 100)
  :group (appearance synergy synergy-bevel))

;;}}}
;;{{{ Auxiliary Functions

(defun brighten-color (color)
  "Brighten a color according to the highlight bevel settings."
  (let* ((red (car (color-rgb color)))
         (green (cadr (color-rgb color)))
         (blue (car (cddr (color-rgb color))))
         (red-delta (quotient (* red synergy-bevel:red-highlight-percentage) 100))
         (green-delta (quotient (* green synergy-bevel:green-highlight-percentage) 100))
         (blue-delta (quotient (* blue synergy-bevel:blue-highlight-percentage) 100)))
    (get-color-rgb (min (+ red-delta red) 65535)
                   (min (+ green-delta green) 65535)
                   (min (+ blue-delta blue) 65535))))

(defun darken-color (color)
  "Darken a color according to the shadow bevel settings."
  (let* ((red (car (color-rgb color)))
         (green (cadr (color-rgb color)))
         (blue (car (cddr (color-rgb color))))
         (red-delta (quotient (* red synergy-bevel:red-shadow-percentage) 100))
         (green-delta (quotient (* green synergy-bevel:green-shadow-percentage) 100))
         (blue-delta (quotient (* blue synergy-bevel:blue-shadow-percentage) 100)))
    (get-color-rgb (max (- red red-delta) 0)
                   (max (- green green-delta) 0)
                   (max (- blue blue-delta) 0))))

;;}}}
;;{{{ Buttons

(defvar close-button (list (make-image "close-inactive.png")
                           (make-image "close.png")
                           nil
                           (make-image "close-clicked.png")))

(defvar minimize-button (list (make-image "minimize-inactive.png")
                              (make-image "minimize.png")
                              nil
                              (make-image "minimize-clicked.png")))

;;}}}
;;{{{ Color Lists

(defvar titlebar-text-color (list synergy:foreground-color-nofocus
                                  synergy:foreground-color-focus))

(defvar titlebar-color (list synergy:background-color-nofocus
                             synergy:background-color-focus))

(defvar titlebar-bevel-highlight-color (list (brighten-color synergy:background-color-nofocus)
                                             (brighten-color synergy:background-color-focus)))

(defvar titlebar-bevel-shadow-color (list (darken-color synergy:background-color-nofocus)
                                          (darken-color synergy:background-color-focus)))

(defvar resizebar-color (list synergy:resizebar-color-nofocus
                              synergy:resizebar-color-focus))

(defvar resizebar-bevel-highlight-color (list (brighten-color synergy:resizebar-color-nofocus)
                                              (brighten-color synergy:resizebar-color-focus)))

(defvar resizebar-bevel-shadow-color (list (darken-color synergy:resizebar-color-nofocus)
                                           (darken-color synergy:resizebar-color-focus)))

;;}}}

(defun title-function ()
  "Returns the correct definition for the titlebar."
  (let* ((real-font (get-font synergy:title-font))
         (text-height (font-height real-font)))
    (if synergy:squeeze-titlebar
        (let* ((title-text-width (lambda (w)
                                   (+ 2 (text-width (window-name w) real-font))))
               (title-shadow-width (lambda (w)
                                     (+ 5 (title-text-width w))))
               (title-highlight-width (lambda (w)
                                        (+ 3 (title-text-width w))))
               (entire-titlebar-width (lambda (w)
                                        (+ 40 (title-text-width w))))
               (highlight-width (lambda (w)
                                  (+ 39 (title-text-width w))))
               (titlebar-width (lambda (w)
                                 (+ 19 (title-text-width w))))
               (title-right-decoration-offset (lambda (w)
                                                (+ 22 (title-text-width w))))
               (close-button-offset (lambda (w)
                                      (+ 2 (title-right-decoration-offset w))))
               (title-end-offset (lambda (w)
                                   (+ 17 (title-right-decoration-offset w)))))
        ; return appropriate magic
        `(; back of the titlebar
         ((background . ,titlebar-color)
          (top-edge . -18)
          (left-edge . 1)
          (width . ,entire-titlebar-width)
          (height . 16)
          (class . title))
         ; where the title is drawn
         ((background . ,titlebar-color)
          (foreground . ,titlebar-text-color)
          (font . ,real-font)
          (text . ,window-name)
          (top-edge . -15)
          (left-edge . 20)
          (width . ,title-text-width)
          (height . ,text-height)
          (class . title))
         ; bottom highlight of the text area
         ((background . ,titlebar-bevel-highlight-color)
          (top-edge . -4)
          (height . 1)
          (left-edge . 17)
          (width . ,title-shadow-width)
          (class . title))
         ; right highlight of the text area
         ((background . ,titlebar-bevel-highlight-color)
          (top-edge . -16)
          (height . ,(+ 2 text-height))
          (left-edge . ,title-right-decoration-offset)
          (width . 1)
          (class . title))
         ; top shadow of the text area
         ((background . ,titlebar-bevel-shadow-color)
          (top-edge . -16)
          (height . 1)
          (left-edge . 18)
          (width . ,title-shadow-width)
          (class . title))
         ; left shadow of the text area
          ((background . ,titlebar-bevel-shadow-color)
           (top-edge . -16)
           (height . ,(1+ text-height))
           (left-edge . 17)
           (width . 1)
           (class . title))
         ; left edge of the titlebar
          ((background . ,synergy:edge-color)
           (left-edge . -1)
           (width . 1)
           (top-edge . -19)
           (height . 19)
           (class . title))
         ; right edge of the titlebar
         ((background . ,synergy:edge-color)
          (left-edge . ,entire-titlebar-width)
          (width . 1)
          (top-edge . -19)
          (height . 19)
          (class . title))
         ; top edge of the titlebar
         ((background . ,synergy:edge-color)
          (left-edge . 0)
          (width . ,entire-titlebar-width)
          (top-edge . -19)
          (height . 1)
          (class . title))
         ; top highlight of the titlebar
         ((background . ,titlebar-bevel-highlight-color)
          (left-edge . 0)
          (width . ,highlight-width)
          (top-edge . -18)
          (height . 1)
          (class . title))
         ; left highlight of the titlebar
         ((background . ,titlebar-bevel-highlight-color)
          (left-edge . 0)
          (width . 1)
          (top-edge . -17)
          (height . 16)
          (class . title))
         ; bottom shadow of the titlebar
         ((background . ,titlebar-bevel-shadow-color)
          (left-edge . 1)
          (width . ,highlight-width)
          (top-edge . -2)
          (height . 1)
          (class . title))
         ; right shadow of the titlebar
         ((background . ,titlebar-bevel-shadow-color)
          (left-edge . ,title-end-offset)
          (width . 1)
          (top-edge . -18)
          (height . 17)
          (class . title))
         ; close button
         ((background . ,titlebar-color)
          (foreground . ,close-button)
          (left-edge . ,close-button-offset)
          (width . 15)
          (top-edge . -17)
          (height . 15)
          (x-justify . center)
          (y-justify . center)
          (cursor . left_ptr)
          (class . close-button))
         ; minimize button
         ((background . ,titlebar-color)
          (foreground . ,minimize-button)
          (left-edge . 1)
          (width . 15)
          (top-edge . -17)
          (height . 15)
          (x-justify . center)
          (y-justify . center)
          (cursor . left_ptr)
          (class . iconify-button))))
      ; else return a normal titlebar
      `(; back of the titlebar
       ((background . ,titlebar-color)
        (top-edge . -18)
        (left-edge . 1)
        (right-edge . 1)
        (height . 16)
        (class . title))
       ; where the title is drawn
       ((background . ,titlebar-color)
        (foreground . ,titlebar-text-color)
        (font . ,real-font)
        (text . ,window-name)
        (top-edge . -15)
        (left-edge . 20)
        (right-edge . 20)
        (height . ,text-height)
        (class . title))
       ; bottom highlight of the text area
       ((background . ,titlebar-bevel-highlight-color)
        (top-edge . -4)
        (height . 1)
        (left-edge . 17)
        (right-edge . 17)
        (class . title))
       ; right highlight of the text area
       ((background . ,titlebar-bevel-highlight-color)
        (top-edge . -16)
        (height . ,(1+ text-height))
        (right-edge . 17)
        (width . 1)
        (class . title))
       ; top shadow of the text area
       ((background . ,titlebar-bevel-shadow-color)
        (top-edge . -16)
        (height . 1)
        (left-edge . 18)
        (right-edge . 18)
        (class . title))
       ; left shadow of the text area
       ((background . ,titlebar-bevel-shadow-color)
        (top-edge . -16)
        (height . ,(1+ text-height))
        (left-edge . 17)
        (width . 1)
        (class . title))
       ; left edge of the titlebar
       ((background . ,synergy:edge-color)
        (left-edge . -1)
        (width . 1)
        (top-edge . -19)
        (height . 19)
        (class . title))
       ; right edge of the titlebar
       ((background . ,synergy:edge-color)
        (right-edge . -1)
        (width . 1)
        (top-edge . -19)
        (height . 19)
        (class . title))
       ; top edge of the titlebar
       ((background . ,synergy:edge-color)
        (right-edge . -1)
        (left-edge . -1)
        (top-edge . -19)
        (height . 1)
        (class . title))
      ; top highlight of the titlebar
      ((background . ,titlebar-bevel-highlight-color)
       (left-edge . 0)
       (right-edge . 1)
       (top-edge . -18)
       (height . 1)
       (class . title))
      ; left highlight of the titlebar
      ((background . ,titlebar-bevel-highlight-color)
       (left-edge . 0)
       (width . 1)
       (top-edge . -17)
       (height . 16)
       (class . title))
      ; bottom shadow of the titlebar
      ((background . ,titlebar-bevel-shadow-color)
       (left-edge . 1)
       (right-edge . 0)
       (top-edge . -2)
       (height . 1)
       (class . title))
      ; right shadow of the titlebar
      ((background . ,titlebar-bevel-shadow-color)
       (right-edge . 0)
       (width . 1)
       (top-edge . -18)
       (height . 17)
       (class . title))
      ; close buttom
      ((background . ,titlebar-color)
       (foreground . ,close-button)
       (right-edge . 1)
       (width . 15)
       (top-edge . -17)
       (height . 15)
       (x-justify . center)
       (y-justify . center)
       (cursor . left_ptr)
       (class . close-button))
      ; minimize button
      ((background . ,titlebar-color)
       (foreground . ,minimize-button)
       (left-edge . 1)
       (width . 15)
       (top-edge . -17)
       (height . 15)
       (x-justify . center)
       (y-justify . center)
       (cursor . left_ptr)
       (class . iconify-button))))))

; these two functions should really be integrated into title-function
(defun title-bottom-edge-normal ()
  `(((background . ,synergy:edge-color)
    (right-edge . 0)
    (left-edge . 0)
    (top-edge . -1)
    (height . 1)
    (class . title))))

(defun title-bottom-edge-shaded ()
  (if synergy:squeeze-titlebar
      (let* ((real-font (get-font synergy:title-font))
             (title-text-width (lambda (w)
                                 (+ 2 (text-width (window-name w) real-font))))
             (entire-titlebar-width (lambda (w)
                                      (+ 40 (title-text-width w)))))
        `(((background . ,synergy:edge-color)
           (left-edge . 0)
           (width . ,entire-titlebar-width)
           (top-edge . -1)
           (height . 1)
           (class . title))))
    `(((background . ,synergy:edge-color)
       (right-edge . 0)
       (left-edge . 0)
       (top-edge . -1)
       (height . 1)
       (class . title)))))

;;{{{ Big Giant Let*

;; frame definition
(let* ((update-text-width (lambda (w)
                            (if (eq (window-get w 'current-frame-style) 'synergy)
                                (rebuild-frame w))))
       ; edges
       (left-edge `((background . ,synergy:edge-color)
                    (left-edge . -1)
                    (width . 1)
                    (top-edge . 0)
                    (bottom-edge . ,(- synergy:resizebar-height))
                    (cursor . left_side)
                    (class . left-border)))
       (right-edge `((background . ,synergy:edge-color)
                     (right-edge . -1)
                     (width . 1)
                     (top-edge . 0)
                     (bottom-edge . ,(- synergy:resizebar-height))
                     (cursor . right_side)
                     (class . right-border)))
       (bottom-edge `((background . ,synergy:edge-color)
                      (left-edge . -1)
                      (right-edge . -1)
                      (bottom-edge . ,(- -1 synergy:resizebar-height))
                      (height . 1)
                      (cursor . bottom_side)
                      (class . bottom-border)))
       ; resize "buttons" -- boxes on left and right sides
       ; FIXME: balance the left and right sides
       ; left "button"
       (lb-highlight-left `((background . ,resizebar-bevel-highlight-color)
                            (left-edge . 0)
                            (width . 1)
                            (bottom-edge . ,(- synergy:resizebar-height))
                            (height . ,synergy:resizebar-height)
                            (class . bottom-left-corner)))
       (lb-highlight-top `((background . ,resizebar-bevel-highlight-color)
                           (left-edge . 1)
                           (width . 23)
                           (bottom-edge . -1)
                           (height . 1)
                           (class . bottom-left-corner)))
       (lb-shadow-right `((background . ,resizebar-bevel-shadow-color)
                          (left-edge . 23)
                          (width . 1)
                          (bottom-edge . ,(- 1 synergy:resizebar-height))
                          (height . ,(- synergy:resizebar-height 2))
                          (class . bottom-left-corner)))
       (lb-shadow-bottom `((background . ,resizebar-bevel-shadow-color)
                           (left-edge . 1)
                           (width . 23)
                           (bottom-edge . ,(- synergy:resizebar-height))
                           (height . 1)
                           (class . bottom-left-corner)))
       (lb-main-box `((background . ,resizebar-color)
                      (left-edge . 1)
                      (width . 22)
                      (bottom-edge . ,(- 1 synergy:resizebar-height))
                      (height . ,(- synergy:resizebar-height 2))
                      (class . bottom-left-corner)))
       ; right "button"
       (rb-highlight-left `((background . ,resizebar-bevel-highlight-color)
                            (right-edge . 22)
                            (width . 1)
                            (bottom-edge . ,(- synergy:resizebar-height))
                            (height . ,(1- synergy:resizebar-height))
                            (class . bottom-right-corner)))
       (rb-highlight-top `((background . ,resizebar-bevel-highlight-color)
                           (right-edge . 1)
                           (width . 22)
                           (bottom-edge . -1)
                           (height . 1)
                           (class . bottom-right-corner)))
       (rb-shadow-right `((background . ,resizebar-bevel-shadow-color)
                          (right-edge . 0)
                          (width . 1)
                          (bottom-edge . ,(- 1 synergy:resizebar-height))
                          (height . ,(1- synergy:resizebar-height))
                          (class . bottom-right-corner)))
       (rb-shadow-bottom `((background . ,resizebar-bevel-shadow-color)
                           (right-edge . 0)
                           (width . 22)
                           (bottom-edge . ,(- synergy:resizebar-height))
                           (height . 1)
                           (class . bottom-right-corner)))
       (rb-main-box `((background . ,resizebar-color)
                      (right-edge . 1)
                      (width . 22)
                      (bottom-edge . ,(- 1 synergy:resizebar-height))
                      (height . ,(- synergy:resizebar-height 2))
                      (class . bottom-right-corner)))
       ; dividers
       (left-divider `((background . ,synergy:edge-color)
                       (left-edge . 24)
                       (width . 1)
                       (bottom-edge . ,(- synergy:resizebar-height))
                       (height . ,synergy:resizebar-height)
                       (class . bottom-border)))
       (right-divider `((background . ,synergy:edge-color)
                        (right-edge . 23)
                        (width . 1)
                        (bottom-edge . ,(- synergy:resizebar-height))
                        (height . ,synergy:resizebar-height)
                        (class . bottom-border)))
       ; main bar
       (bar-main `((background . ,resizebar-color)
                   (left-edge . 25)
                   (right-edge . 24)
                   (bottom-edge . ,(- 1 synergy:resizebar-height))
                   (height . ,(- synergy:resizebar-height 2))
                   (class . bottom-border)))
       (bar-highlight-top `((background . ,resizebar-bevel-highlight-color)
                            (left-edge . 25)
                            (right-edge . 24)
                            (bottom-edge . -1)
                            (height . 1)
                            (class . bottom-border)))
       (bar-shadow-bottom `((background . ,resizebar-bevel-shadow-color)
                            (left-edge . 25)
                            (right-edge . 24)
                            (bottom-edge . ,(- synergy:resizebar-height))
                            (height . 1)
                            (class . bottom-border)))
       (normal-frame `(,bar-main
                      ,bar-highlight-top
                      ,bar-shadow-bottom
                      ,left-divider
                      ,right-divider
                      ,rb-main-box
                      ,rb-shadow-right
                      ,rb-shadow-bottom
                      ,rb-highlight-top
                      ,rb-highlight-left
                      ,lb-main-box
                      ,lb-shadow-right
                      ,lb-shadow-bottom
                      ,lb-highlight-top
                      ,lb-highlight-left
                      ,left-edge
                      ,right-edge
                      ,bottom-edge
                      ,@(title-function)
                      ,@(title-bottom-edge-normal)))
       (shaded-frame `(,@(title-function)
                         ,@(title-bottom-edge-shaded))))
  (add-frame-style 'synergy
                   (lambda (w type)
                     (case type
                       ((default) normal-frame)
                       ((shaded) shaded-frame)
                       ((transient) normal-frame)
                       ((shaded-transient) shaded-frame)
                       ((unframed) nil-frame)))))

;;}}}

; make sure that the title width gets updated when the name changes
(defun update-text-width (w)
  (if (eq (window-get w 'current-frame-style)
          'synergy)
      (rebuild-frame w)))

(call-after-property-changed 'WM_NAME update-text-width)
